;+
; NAME:
;	sxt_dstreek
; PUPROSE:
;	To remove bad pixels in streaks due to cosmic ray hits.
;	The default parameters are optimized for prepared, compressed
;	SXT images.
; CALLING SEQUENCE:
;	result = sxt_dstreek(image [,sens=sens,thresh=thresh,pix=pix]
; INPUTS:
;	image = 2-dimensional image
; OPTIONAL KEYWORD INPUTS:
;   	sens = sensitivity to streaks
;   		(sens = 10 is typical for "compressed" sxt image)
;   	thresh = threshold for second-pass pixel map
;   		(thresh = 20.0 is typical for "compressed" sxt image)
;	pix = show good_pixmap and final sfd_compressed image.
; PROCEDURE:
;	Image should be either:
;		1) An SFD image, or
;		2) An sxt_prep'd, sxt_kleen'd and, if desired an
;			sxt_composite'd image.
;	Program first prepares an SFD, then,
;
;	Streaks at various angles are amplified and located using
;	convolution and thresholding. This information is used to generate
;	a map of good (undamaged) pixels. Bad pixels are replaced with a 
;	weighted average of nearby good pixels. In a second pass, the
;	result of this operation is subtracted from the original image.
;	The good pixel map is revised based on the difference image,
;	and pixel replacement is done again.
;
;	If the input image was not an SFD (byte-type) then the streak
;	pixels in the original image are replaced with sfd_decomp'd
;	pixels from the destreaked SFD.
; RESTRICTIONS:
;	The program does not work well on images which have not 
;	been sxt_prep'd.
; HISTORY
;	Original destreaking scheme invented by Charles Kankelborg.
;	20-Apr-98, LWA	Genralized for use with non-SFD images.
;       22-Sep-98, TRM  Added check that there are bad pixels before trying to
;                       replace them.
;-

pro convol_enhance, img, ker, goodpix, sens
;Convolve img with kernel ker in all four possible orientations.
;Use this to update the good pixel map.

for i=0, 1 do begin  ;used to go to 3, back when z was zero.
	k = rotate(ker,i)/total(abs(ker))
	enhanced = convol(img,k)   ;/(smoothed+(5*sens))
	good = float(100.0 GT (sens*enhanced))
	goodpix = goodpix AND good
endfor

end

;=========================

function sxt_dstreek, image_in, sens=sens, thresh=thresh, pix=pix

if keyword_set(thresh) then threshold = thresh $
	else threshold = 15.0 ;a reasonable default

if keyword_set(sens) then sensitivity = sens $
	else sensitivity = 15.0 ; a reasonable default
	
case 1 of
   data_chk(image_in, /type) eq 1 : begin  ;Check for byte-type data
      image=image_in
   endcase
   else : image=sfd_comp(image_in)     ;Prepare unnormalized SFD.
endcase

epsilon = 0.1

image=float(image)+epsilon

; (1) Make map of good pixels.

; Find streaks.
a = 1.0
b = -1.0/4.0 ;formerly -1.0/3.0
z = b  ;formerly 0.0
good_pixmap = float(image GT 0.0) ;initialize good pixel map

kernel=[[b,b,b,b,b], $
        [b,b,b,b,b], $
        [a,a,a,a,a], $
        [z,z,z,z,z], $
        [b,b,b,b,b]]

convol_enhance, image, kernel, good_pixmap, sensitivity

kernel=[[b,b,b,b,b], $
        [b,b,b,b,a], $
        [b,a,a,a,z], $
        [a,z,z,z,b], $
        [z,b,b,b,b]]

convol_enhance, image, kernel, good_pixmap, sensitivity        

kernel=transpose(kernel)     
          
convol_enhance, image, kernel, good_pixmap, sensitivity          
     
kernel=[[b,b,b,b,b], $
        [b,b,b,a,a], $
        [b,b,a,z,z], $
        [a,a,z,b,b], $
        [z,z,b,b,b]]

convol_enhance, image, kernel, good_pixmap, sensitivity

kernel=transpose(kernel)     
          
convol_enhance, image, kernel, good_pixmap, sensitivity          
     
kernel=[[z,b,b,b,a], $
        [b,b,b,a,z], $
        [b,b,a,z,b], $
        [b,a,z,b,b], $
        [a,z,b,b,b]]
        
convol_enhance, image, kernel, good_pixmap, sensitivity

if keyword_set(pix) then begin
   siz=size(image_in)
   wdef,2,siz(1)
   tvscl,good_pixmap
endif

; (2) Replace bad pixels.
newpix = ck_smooth(good_pixmap*image,3)/ck_smooth(good_pixmap,3) >0 <image
result = good_pixmap*image + (NOT good_pixmap)*newpix

; (3) Second pass
good_pixmap = float((image - result) LT threshold)
newpix = ck_smooth(good_pixmap*image,3)/ck_smooth(good_pixmap,3) >0 <image
result = good_pixmap*image + (NOT good_pixmap)*newpix - epsilon

; (4) For non-SFD replace bad pixels with pixels from RESULT.
case 1 of
   data_chk(image_in, /type) eq 1 : begin  ;Return SFD form.
      image=byte(result)
   endcase
   else : begin                        ;Decompress unnormalized SFD
      result=sfd_decomp(byte(result))
      ii=where(good_pixmap eq 0)
      image=image_in
      if ii(0) GE 0 then image(ii)=result(ii)  ; TRM added ii[0] check 
   endelse
endcase

if keyword_set(pix) then begin
   wdef,0,siz(1)
   case 1 of
      data_chk(image_in, /type) eq 1 : begin
         tvscl,image
      endcase
      else : tvscl,sfd_comp(image)
   endcase
endif

return,image

END

